/////////////////////////////////////////////////////////////////////
//
// dsp56k.c
//
// handles communication to dsk board
//
// Date:		25.10.2009
// Author:		fcordes@cs.tu-berlin.de
//
/////////////////////////////////////////////////////////////////////

#include "util/delay.h"
#include "avr/io.h"
#include "avr/eeprom.h"
#include "string.h"
#include "usart_debug.h"
#include "dsp56k.h"
#include "com_def.h"

#include "usb_drv.h"
#include "stdio.h"

dspsys_host_message_t buf;
uint8_t	bufferLock;

#define LOCKED 1
#define UNLOCKED 0

#define BUFFER_LOCKED bufferLock == LOCKED
#define BUFFER_UNLOCKED bufferLock == UNLOCKED
#define LOCK_BUFFER() bufferLock = LOCKED
#define UNLOCK_BUFFER() bufferLock = UNLOCKED


//#define __DEBUG_READDEFAULTVALUES__
//#define __DEBUG_REDIRECT__

#define XMEMSIZE 256
#define YMEMSIZE 256
#define PMEMSIZE 512

uint32_t x_mem[XMEMSIZE] EEMEM; // Buffers for X, Y, and P memory
uint32_t y_mem[YMEMSIZE] EEMEM;
uint32_t p_mem[PMEMSIZE] EEMEM;



static void dspsysUsbDevReadBytesN(void *c, uint8_t n);
static void dspsysUsbDevWriteBytesN(void *c, uint8_t n);
void dspsysStartTransaction( uint8_t boottime );
uint8_t dspsysWaitTXDE ( void );
uint8_t dspsysWaitRXDF ( void );

void dspsysInitIO( void )
{
	// Setup IO Ports
	Debug("Seting up I/O Ports...");
	// D_PORT can be Input/Output, this will be setup each time it is used
	// Init to input and disable pullups
	D_PORT_DDR = 0x00;
	D_PORT = 0x00;

	// A_PORT is output only
	A_PORT_DDR = 0xFF;

	// JP_PORT is input only with pullups enabled exept for JP3 which
	// is HEN
	JP_PORT_DDR = 0x00 | (1<<JP3);
	JP_PORT = ((1<<JP0) | (1<<JP1) | (1<<JP2));

	// JP3 is the reset line which should be active (LO) at boot time
	JP_PORT &= ~(1<<JP3);
	Debug("done!\r\n");
}

void dspsysDecodeMessage(dspsys_host_message_t *pMsg)
{
//	uint16_t i;
//	uint8_t bytesInFIFO = 0;
	char debugString[60];
	uint8_t data = 0;


/*
	for (i=0; i<16; i++)
	{
			
		sprintf(debugString, "$%u=[0,%u,%u,%u]\r\n", 
			i, 
			((uint8_t*)pMsg)[(i*sizeof(uint32_t))+2],
			((uint8_t*)pMsg)[(i*sizeof(uint32_t))+1],
			((uint8_t*)pMsg)[(i*sizeof(uint32_t))+0] );
		Debug(debugString);
	}

	sprintf(debugString, "pMsg->type=%u\r\n", pMsg->type);
	Debug(debugString);
	sprintf(debugString, "pMsg->addr=%u\r\n", pMsg->addr);
	Debug(debugString);
	sprintf(debugString, "pMsg->count=%u\r\n", pMsg->count);
	Debug(debugString);
	sprintf(debugString, "pMsg->flags=%u\r\n", pMsg->flags);
	Debug(debugString);

*/


	



	// Analyze type of access and resulting payload
	switch (pMsg->type)
	{
		case DSPSYS_WRITE_P:
			Debug("DSPSYS_WRITE_P\r\n");
			dspsysStartTransaction(0);
			// Write to Program mem
			dspsysWriteMemoryBlock(
				CMD_WP, 
				pMsg->addr, 
				pMsg->data, 
				pMsg->count, 
				pMsg->flags);
			break;

		case DSPSYS_WRITE_X:
			Debug("DSPSYS_WRITE_X\r\n");
			dspsysStartTransaction(0);
			// Write to X mem
			dspsysWriteMemoryBlock(
				CMD_WX, 
				pMsg->addr, 
				pMsg->data, 
				pMsg->count, 
				pMsg->flags);
			break;

		case DSPSYS_WRITE_Y:
			Debug("DSPSYS_WRITE_Y\r\n");
			dspsysStartTransaction(0);
			// Write to Y mem
			dspsysWriteMemoryBlock(
				CMD_WY, 
				pMsg->addr, 
				pMsg->data, 
				pMsg->count, 
				pMsg->flags);
			break;

		case DSPSYS_READ_P:
			Debug("DSPSYS_READ_P\r\n");
			dspsysStartTransaction(0);
			// Read from P mem, copy to FIFO
			dspsysReadMemoryBlock(
				CMD_RP, 
				pMsg->addr, 
				pMsg->data, 
				pMsg->count, 
				pMsg->flags);

			// Submit to FIFO
			dspsysCopyToFIFO(pMsg);
			break;
	
		case DSPSYS_READ_X:
			Debug("DSPSYS_READ_X\r\n");
			dspsysStartTransaction(0);
			// Read from X mem
			dspsysReadMemoryBlock(
				CMD_RX, 
				pMsg->addr, 
				pMsg->data, 
				pMsg->count, 
				pMsg->flags);

			// Submit to FIFO
			dspsysCopyToFIFO(pMsg);
			break;

		case DSPSYS_READ_Y:
			Debug("DSPSYS_READ_Y\r\n");
			dspsysStartTransaction(0);
			// Read from Y mem
			dspsysReadMemoryBlock(
				CMD_RY, 
				pMsg->addr, 
				pMsg->data, 
				pMsg->count, 
				pMsg->flags );
			// Submit to FIFO
			dspsysCopyToFIFO(pMsg);
			break;

		case DSPSYS_WRITE_DIRECT:
			Debug("DSPSYS_WRITE_DIRECT\r\n");
	
			dspsysStartTransaction(0);
			// Set control lines
			data = 0;
			
			dspsysWriteRegister (
				pMsg->addr,
				(uint8_t*)pMsg->data );
			break;

		case DSPSYS_READ_DIRECT:
			Debug("DSPSYS_READ_DIRECT\r\n");
			dspsysStartTransaction(0);
			// Read control lines
			pMsg->data[0] = 0;

			data = 0;

			dspsysReadRegister (
				pMsg->addr,
				(uint8_t*)(pMsg->data));

			// Submit to FIFO
			dspsysCopyToFIFO(pMsg);
			break;

		case DSPSYS_RESET_ON:
			Debug("DSPSYS_RESET_ON\r\n");
			// Activate reset line - no payload
			dspsysActivateReset();
			break;

		case DSPSYS_RESET_OFF:
			Debug("DSPSYS_RESET_OFF\r\n");
			// Deactivate reset line - no payload
			dspsysDeactivateReset();
			dspsysStartTransaction(1);
			break;

		default:
			// Unknown, signal failure
			break;
	}

	// unlock buffer for next packet
	UNLOCK_BUFFER();
}


void dspsysStartTransaction( uint8_t boottime )
{
	uint8_t timeout;
	char debugString[80];
	uint8_t icr;
	uint8_t data = 0;
	
	
	if (boottime!=0)
	{
		data = 0;
		dspsysWriteRegister(ICR, &data);
		data = 1;
		dspsysWriteRegister(CVR, &data);
		data = 3;
		dspsysWriteRegister(IVR, &data);
	}
	timeout = 10;
	dspsysReadRegister(ISR, &data);
	//sprintf(debugString, "* ISR = %u\r\n", data);
	//Debug(debugString);

	if (data&(1<<7))
	{
		//Debug("HREQ bit is set\r\n");
		dspsysReadRegister(ICR, &icr);
		if (icr != 0)
		{
			Debug("ICR!=0, cleared\r\n");
			icr = 0;
			dspsysWriteRegister(ICR, &icr);
		}
		while ((data&(1<<7)) && ((timeout--) > 0))
		{
			if (data & (1<<RXDF))
			{
				//Debug("RXDF bit is set - cleaning registers\r\n");
				dspsysReadRegister(RXL, &data);
				dspsysReadRegister(RXL, &data);
			}
			_delay_ms(1);
			dspsysReadRegister(ISR, &data);
			//sprintf(debugString, "* ISR = %u\r\n", data);
			//Debug(debugString);	
		}
	}
	if (timeout == 0)
	{ 
		Debug("Operation timed out\r\n");
	}
}
			

void dspsysActivateReset( void )
{		
	// Make sure JP3 is output and set it to LOW
	JP_PORT_DDR |= (1<<JP3);
	JP_PORT &= ~(1<<JP3);

	return;
}


void dspsysDeactivateReset( void )
{
	uint8_t data;
	uint8_t timeout;
//	char debugString[80];

	// Disconnect JP3 (should be input)
	JP_PORT_DDR &= ~(1<<JP3);
	JP_PORT |= (1<<JP3);

	// Give the device a little while to start up
	_delay_ms(1); 	

	// write the init bit with RREQ enabled
	//data = (1<<INIT);
	//dspsysWriteRegister(ICR, &data);

	return;
}


dspsys_host_message_t* dspsysCopyFIFO (void)
{
	uint8_t i;
	uint8_t oldEndpointNumber;
	uint16_t byteCount;

	oldEndpointNumber = UsbDevGetCurrentEndpoint();
	UsbDevSelectEndpoint(3);

	// Do not proceed if buffer is still in use
	if (BUFFER_LOCKED)
	{
		Debug("Unable to copy FIFO because buffer is locked\r\n");
		return 0;
	}
	
	// neither do if FIFO does not hold enough data
	byteCount=UsbDevGetByteCountLow();
	if (byteCount < sizeof(dspsys_host_message_t))
	{
		char debugString[40];
		sprintf(debugString, "(%u bytes avl.\r\b", byteCount);
		Debug("dspsys: buffer underrun reading FIFO ");
		Debug(debugString);
		return 0;
	}

	// purge old content from buffer
	for (i=0; i<sizeof(dspsys_host_message_t); i++)
	{
		((uint8_t*)&buf)[i]= 0;
	}

	// copy contents of FIFO into buffer
	dspsysUsbDevReadBytesN(&buf, sizeof(dspsys_host_message_t));

	// lock buffer - this will be cleared when the actions contained in this
	// package have executed. Until then, do not accept new data.
	LOCK_BUFFER();

	// restore previously selected endpoint
	UsbDevSelectEndpoint(oldEndpointNumber);

	return &buf;
}

void dspsysCopyToFIFO (dspsys_host_message_t *pMsg)
{
//	uint8_t i;
	uint8_t oldEndpointNumber;

	// Submit to FIFO
	oldEndpointNumber = UsbDevGetCurrentEndpoint();
	UsbDevSelectEndpoint(2);
	// Clear TXINI
	UsbDevClearTransmitterReady();

	// copy contents of FIFO into buffer
	if (UsbDevGetByteCountLow() == 0)
	{
		dspsysUsbDevWriteBytesN(&buf, sizeof(dspsys_host_message_t));
	} else
	{
			Debug("FIFO not empty\r\n");
	}
	// Clear FIFOCON
	UsbDevSendInData();

	// Restore previously selected endpoint	
	UsbDevSelectEndpoint(oldEndpointNumber);

	return ;
}


// Read n bytes from FIFO; FIFO should contain exactly n bytes
// Limited to n < 256 <--- Thats okay, we only have 64 byte packets
// C&P S.Salewski
static void
dspsysUsbDevReadBytesN(void *c, uint8_t n)
{
  Assert(UsbDevGetByteCountLow() == n);
  while (n--) *(uint8_t*)c++ = UsbDevReadByte();
}

static void 
dspsysUsbDevWriteBytesN(void *c, uint8_t n)
{
  uint8_t i = 0;
  Assert(UsbbDevGetByteCountLow() == 0);
  while (n--) 
  { 
  	UsbDevWriteByte(((uint8_t*)c)[i]);
	i++;
  }
}

void dspsysSetAddr(uint8_t addr)
{
	// mask out upper 5 bits of addr
	addr&= ((1<<A0) | ( 1<<A1) | (1<<A2));
	// copy these from current A_PORT value
	addr|= (A_PORT & ((1<<PA3) | (1<<PA4) | (1<<PA5) | (1<<PA6) | (1<<PA7)));
	// set new value
	A_PORT = addr;

	return;
}

void dspsysSetData(uint8_t data)
{

	// Make sure data port is output
	D_PORT_DDR = 0xFF;

	// output data
	D_PORT = data;

	// waste time
	return;
}

uint8_t dspsysGetData (void)
{

	// Make sure data port is input with pullups switched off
	D_PORT_DDR = 0x00;
	D_PORT = 0x00;

	// waste time
	_delay_ms(1);

	// return data
	return D_PIN;
}


void dspsysWriteRegister(uint8_t reg, uint8_t* data)
{
	char debugString[20];
	// setup write access
	HP_WRITE_ENABLE();
	HP_SETUP_ACCESS();

	// Setup address & data
	dspsysSetAddr(reg);
	dspsysSetData(*data);

	// Strobe HEN
	HP_PERFORM_ACCESS();

		sprintf(debugString, "Unknown target\r\n");
	switch(reg)
	{
		case ICR:
			sprintf(debugString, "Wrote ICR=%u\r\n", *data);
			break;
		case CVR:
			sprintf(debugString, "Wrote CVR=%u\r\n", *data);
			break;
		case ISR:			
			sprintf(debugString, "Wrote ISR=%u\r\n", *data);
			break;
		case IVR:
			sprintf(debugString, "Wrote IVR=%u\r\n", *data);
			break;
			
		case TXH:
			sprintf(debugString, "Wrote TXH=%u\r\n", *data);
			break;
		case TXM:
			sprintf(debugString, "Wrote TXM=%u\r\n", *data);
			break;
		case TXL:
			sprintf(debugString, "Wrote TXL=%u\r\n", *data);
			break;

		default:
			sprintf(debugString, "Wrote ICR=%u\r\n", *data);
			break;
	}
	Debug(debugString);


	return;
}

void dspsysReadRegister(uint8_t reg, uint8_t* data)
{
	char debugString[20];
	// Proceed only with valid pointer
	if (data == 0)
	{
		Debug("Invalid pointer passed to dspsysReadRegister");
		return;
	}
	// setup read access
	HP_READ_ENABLE();

	// setup address
	dspsysSetAddr(reg);

	// Strobe HEN
	HP_SETUP_ACCESS();
		
	// Read data
#ifndef __DEBUG_READDEFAULTVALUES__
	*data = dspsysGetData();
#endif

#ifdef __DEBUG_READDEFAULTVALUES__
	Debug("Debug mode: reading default values\r\n");
	switch(reg)
	{
		case ICR:
			*data = 0x00;
			break;
		case CVR:
			*data = 0x12;
			break;
		case ISR:
			*data = 0x06;
			break;
		case IVR:
			*data = 0x0F;
			break;
			
		default:
			*data = 0x00;
			break;
	}
#endif

	HP_PERFORM_ACCESS();


	sprintf(debugString, "Unknown target\r\n");
	switch(reg)
	{
		case ICR:
			sprintf(debugString, "Read ICR=%u\r\n", *data);
			break;
		case CVR:
			sprintf(debugString, "Read CVR=%u\r\n", *data);
			break;
		case ISR:
			//sprintf(debugString, "unmodified %u\r\n", *data);
			//Debug(debugString);
			
			//*data &= 0b01111111; // Erase HREQ
			
			sprintf(debugString, "Read ISR=%u\r\n", *data);
			break;
		case IVR:
			sprintf(debugString, "Read IVR=%u\r\n", *data);
			break;
			
		case RXH:
			sprintf(debugString, "Read RXH=%u\r\n", *data);
			break;
		case RXM:
			sprintf(debugString, "Read RXM=%u\r\n", *data);
			break;
		case RXL:
			sprintf(debugString, "Read RXL=%u\r\n", *data);
			break;
		default:
			sprintf(debugString, "Read ICR=%u\r\n", *data);
			break;
	}
	Debug(debugString);



	return;
}


#ifndef __DEBUG_REDIRECT__
void dspsysWriteMemory(uint8_t target, uint32_t addr, uint32_t* data)
{
	
	inttype_convert_t address;
	inttype_convert_t datablock;
	uint8_t temp;

	address.int32 = addr;
	datablock.int32 = *data;


	// To issue [Read/Write] to [X/Y/P] memory command, a two-stage transfer is
	// needed
	// First, setup address by calling the appropiate ISR
	// Write address into TXL, TXM
	dspsysWriteRegister(TXM, &(address.int8[1]));
	dspsysWriteRegister(TXL, &(address.int8[0]));
	
	// Call ISR to setup following transfer
	temp = ( target | (1<<HC_BIT) );
	dspsysWriteRegister(CVR, &temp);

	// Maybe it is neccessary to wait here for a little while
	// Second stage is a call to the ISR that actually performs the transfer
	// Write lower three bytes into TXL, TXM, TXH, ignore the fourth
	// write to TXL last, since writing to it triggers the processing
	// Wait for TXDE
	dspsysWaitTXDE ();
	// Write data 
	dspsysWriteRegister(TXH, &(datablock.int8[2]));
	dspsysWriteRegister(TXM, &(datablock.int8[1]));
	dspsysWriteRegister(TXL, &(datablock.int8[0]));

	// Call ISR to perform transfer
	temp = ( ISR_HRD | (1<<HC_BIT) );
	dspsysWriteRegister(CVR, &temp);

	return;
}
#endif

#ifdef __DEBUG_REDIRECT__
void dspsysWriteMemory(uint8_t target, uint32_t addr, uint32_t* data)
{
	Debug("Debug mode: Redirected to EEPROM\r\n");
	addr = addr % 0x300;

	char debugString[80];
	switch (target)
	{
		case CMD_WP:
			if (addr < PMEMSIZE)
			{
				eeprom_write_dword(&(p_mem[addr]), *data);
			} else {
				sprintf(debugString, "Invalid address(%i)\r\n", addr);
				Debug (debugString);
			}
			break;
			
		case CMD_WX:
			if (addr < XMEMSIZE)
			{
				eeprom_write_dword(&(x_mem[addr]), *data);
			} else {
				sprintf(debugString, "Invalid address(%i)\r\n", addr);
				Debug (debugString);
			}
			break;

		case CMD_WY:
			if (addr < YMEMSIZE)
			{
				eeprom_write_dword(&(y_mem[addr]), *data);
			} else {
				sprintf(debugString, "Invalid address(%i)\r\n", addr);
				Debug (debugString);
			}
			break;
		
		default:
			sprintf(debugString, "Unknown target(%i)\r\n", target);
			Debug (debugString);
			break;
	}

	
	sprintf(debugString, "W: $%i<-[%i,%i,%i]\r\n", 
			(uint16_t)addr, 
			((uint8_t*)data)[2],
			((uint8_t*)data)[1],
			((uint8_t*)data)[0] );
    Debug(debugString);
	return;
}
#endif

#ifndef __DEBUG_REDIRECT__
void dspsysReadMemory(uint8_t target, uint32_t addr, uint32_t* data)
{
	inttype_convert_t address;
	inttype_convert_t datablock;
	uint8_t temp;

	address.int32 = addr;
	

	// To issue [Read/Write] to [X/Y/P] memory command, a two-stage transfer is
	// needed
	// First, setup address by calling the appropiate ISR
	// Write address into TXL, TXM
	
	dspsysWriteRegister(TXM, &(address.int8[1]));
	dspsysWriteRegister(TXL, &(address.int8[0]));
	// Call ISR to setup following transfer
	temp = (target | (1<<HC_BIT));
	dspsysWriteRegister(CVR, &temp);

	// Call ISR to perform transfer
	temp = (ISR_HTD | (1<<HC_BIT));
	dspsysWriteRegister(CVR, &temp);

	// Maybe it is neccessary to wait here for a little while
	// Second stage is a call to the ISR that actually performs the transfer
	// Read RYL, RXM, RXH into datablock, do not read RXL until everything else
	// has been read 
	// Wait for RDXF
	dspsysWaitRXDF();

	dspsysReadRegister(RXH, &(datablock.int8[2]));
	dspsysReadRegister(RXM, &(datablock.int8[1]));
	dspsysReadRegister(RXL, &(datablock.int8[0]));


	// return read values
	
	memcpy(data, &datablock, sizeof(uint32_t));
	return;
}
#endif

#ifdef __DEBUG_REDIRECT__
void dspsysReadMemory(uint8_t target, uint32_t addr, uint32_t* data)
{
	char debugString[80];
	
	Debug("Debug mode: Redirected to EEPROM\r\n");
	addr = addr % 0x300;

	switch (target)
	{
		case CMD_RP:
			if (addr < PMEMSIZE)
			{
				*data = eeprom_read_dword(&(p_mem[addr]));
			} else {
				sprintf(debugString, "Invalid address(%i)\r\n", addr);
				Debug (debugString);
			}
			break;
			
		case CMD_RX:
			if (addr < XMEMSIZE)
			{
				*data = eeprom_read_dword(&(x_mem[addr]));
			} else {
				sprintf(debugString, "Invalid address(%i)\r\n", addr);
				Debug (debugString);
			}
			break;

		case CMD_RY:
			if (addr < YMEMSIZE)
			{
				*data = eeprom_read_dword(&(y_mem[addr]));
			} else {
				sprintf(debugString, "Invalid address(%i)\r\n", addr);
				Debug (debugString);
			}
			break;
		
		default:
			
			sprintf(debugString, "Unknown target(%i)\r\n", target);
			Debug (debugString);
			break;
	}


	sprintf(debugString, "R: $%i->[%i,%i,%i]\r\n", 
			(uint16_t)addr, 
			((uint8_t*)data)[2],
			((uint8_t*)data)[1],
			((uint8_t*)data)[0] );
    Debug(debugString);

	return;
}
#endif

void dspsysWriteMemoryBlock(uint8_t target, uint32_t addr, uint32_t* data, uint32_t count, uint8_t flags)
{
//	char debugString[80];
//	uint8_t k;
	uint8_t i = 0;
/*
	sprintf(debugString, "Writing block [target=%u, addr=%u, count=%u, flags=%u]\r\n", target, addr, count, flags);
	Debug(debugString);


	for (k=0; k<count;k++)
	{
		sprintf(debugString, "%u: %u, %u, %u, %u\r\n",
				k,
				((uint8_t*)data)[k*sizeof(uint32_t)+3],
				((uint8_t*)data)[k*sizeof(uint32_t)+2],
				((uint8_t*)data)[k*sizeof(uint32_t)+1],
				((uint8_t*)data)[k*sizeof(uint32_t)] );
		Debug(debugString);
	}
	Debug("=== End of Dump ===\r\n");
*/
	if (count==0)
	{
		Debug("Count of 0 passed to dspsysWriteMemoryBlock()\r\n");
	} else 
	{
	
		// write count dwords to dsp mem
		while(i < count)
		{
#ifndef __DEBUG_REDIRECT__
			dspsysStartTransaction(0);
#endif
			dspsysWriteMemory(target, addr, &(data[i]));
			i++;
			if (flags & INCREMENTAL_ADDRESSING)
			{
				addr++;
			}
		}
	}
	return;	
}


void dspsysReadMemoryBlock(uint8_t target, uint32_t addr, uint32_t* data, uint32_t count, uint8_t flags)
{
//	char debugString[80];
//	uint8_t k;
	uint8_t i = 0;
/*
	sprintf(debugString, "Reading block [target=%u, addr=%u, count=%u, flags=%u]\r\n", target, addr, count, flags);
	Debug(debugString);
*/
	if (count==0)
	{
		Debug("Count of 0 passed to dspsysReadMemoryBlock()\r\n");
	} else
	{
		// read count dwords from dsp mem
		while(i < count)
		{

#ifndef __DEBUG_REDIRECT__
			dspsysStartTransaction(0);
#endif
			dspsysReadMemory(target, addr, &(data[i]));
			i++;
			if (flags & INCREMENTAL_ADDRESSING)
			{
				addr++;
			}
		}

/*
		for (k=0; k<count;k++)
		{
			sprintf(debugString, "%u: %u, %u, %u, %u\r\n",
					k,
					((uint8_t*)data)[k*sizeof(uint32_t)+3],
					((uint8_t*)data)[k*sizeof(uint32_t)+2],
					((uint8_t*)data)[k*sizeof(uint32_t)+1],
					((uint8_t*)data)[k*sizeof(uint32_t)] );
			Debug(debugString);
		}
		Debug("=== End of Dump ===\r\n");
*/
	}

	return;
}




void dspsysDebugHostport(void)
{
	char debugString[80];

	if (A_PORT & (1<<HRW))
	{
		sprintf(debugString, "Read: addr=%i, data=%i\r\n", (A_PORT & (0b00000111)), D_PORT);
	} else
	{
		sprintf(debugString, "Write: addr=%i, data=%i\r\n", (A_PORT & (0b00000111)), D_PORT);
	}
	Debug(debugString);
}

uint8_t dspsysWaitTXDE ( void )
{
	uint8_t temp;
	uint8_t timeout=10;

	dspsysReadRegister(ISR, &temp);
	while (!(temp&(1<<TXDE)) & (timeout > 0))
	{
		_delay_ms(1);
		dspsysReadRegister(ISR, &temp);
		timeout--;
	}
	if (timeout == 0)
	{
		Debug("Waiting for TXDE timed out...\r\n");
		return 0;
	}
	return 1;
}

uint8_t dspsysWaitRXDF ( void )
{
	uint8_t temp;
	uint8_t timeout=10;

	dspsysReadRegister(ISR, &temp);
	while (!(temp&(1<<RXDF)) && (timeout > 0))
	{
		_delay_ms(1);
		dspsysReadRegister(ISR, &temp);
		timeout--;
	}
	if (timeout == 0)
	{
		Debug("Waiting for RXDF timed out...\r\n");
		return 0;
	}
	return 1;
}
